@@ -8,11 +8,13 @@
  */
 
 #include <jni.h>
+#include <stdlib.h>
 #include <cairo.h>
 #include <jg_jnu.h>
 
 #ifndef _Included_org_freedesktop_cairo_Context
 #define _Included_org_freedesktop_cairo_Context
+#include "org_freedesktop_cairo_Context.h"
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -132,7 +134,7 @@ JNIEXPORT void JNICALL Java_org_freedesktop_cairo_Context_cairo_1set_1source_1su
 	cairo_surface_t *surface;
 
 	cr = (cairo_t*)getPointerFromHandle(env, obj);
-	surface = (cairo_surface_t*)getPointerFromHandle(env, surface);
+	surface = (cairo_surface_t*)getPointerFromHandle(env, sur);
 	cairo_set_source_surface(cr, surface, sx, sy);
 }
 
@@ -776,11 +778,10 @@ JNIEXPORT jobject JNICALL Java_org_freedesktop_cairo_Context_cairo_1get_1font_1m
   (JNIEnv *env, jclass cls, jobject obj)
 {
 	cairo_t *cr;
-	cairo_matrix_t matrix;
-
+	cairo_matrix_t *matrix = malloc (sizeof (cairo_matrix_t));
 	cr = (cairo_t*)getPointerFromHandle(env, obj);
-	cairo_get_font_matrix(cr, &matrix);
-	return getHandleFromPointer(env, &matrix);
+	cairo_get_font_matrix(cr, matrix);
+	return getHandleFromPointer(env, matrix);
 }
 
 /*
@@ -1032,15 +1033,17 @@ JNIEXPORT jdouble JNICALL Java_org_freedesktop_cairo_Context_cairo_1get_1miter_1
 /*
  * Class:     org_freedesktop_cairo_Context
  * Method:    cairo_get_matrix
- * Signature: (Lorg/gnu/glib/Handle;Lorg/gnu/glib/Handle;)V
+ * Signature: (Lorg/gnu/glib/Handle;)Lorg/gnu/glib/Handle;
  */
-JNIEXPORT void JNICALL Java_org_freedesktop_cairo_Context_cairo_1get_1matrix
-  (JNIEnv *env, jclass cls, jobject obj, jobject matrix)
+JNIEXPORT jobject JNICALL Java_org_freedesktop_cairo_Context_cairo_1get_1matrix
+  (JNIEnv *env, jclass cls, jobject obj)
 {
-	cairo_t *cr = (cairo_t*)getPointerFromHandle(env, obj);
-	cairo_matrix_t *matrix_g = (cairo_matrix_t*)getPointerFromHandle(env, matrix);
-	cairo_get_matrix(cr, matrix_g);
-	updateHandle(env, matrix, matrix_g);
+	cairo_t *cr;
+	cairo_matrix_t *matrix;
+	matrix = malloc (sizeof (cairo_matrix_t));
+	cr = (cairo_t*) getPointerFromHandle(env, obj);
+	cairo_get_matrix(cr, matrix);
+	return getHandleFromPointer(env, matrix);
  }
 
 /*
@@ -1117,8 +1120,7 @@ JNIEXPORT jobject JNICALL Java_org_freedesktop_cairo_Context_cairo_1get_1font_1o
   (JNIEnv *env, jclass cls, jobject obj)
 {
 	cairo_t *cr = (cairo_t*)getPointerFromHandle(env, obj);
-	cairo_font_options_t *fo;
-	
+	cairo_font_options_t *fo = cairo_font_options_create();
 	cairo_get_font_options(cr, fo);
 	return getHandleFromPointer(env, fo);
 }
